@@ -1,18 +1,19 @@
 """Code for loading and manipulating data structures."""
-from .inference_data import InferenceData, concat
-from .io_netcdf import from_netcdf, to_netcdf
-from .datasets import load_arviz_data, list_datasets, clear_data_home
-from .base import numpy_to_data_array, dict_to_dataset, CoordSpec, DimSpec
+from .base import CoordSpec, DimSpec, dict_to_dataset, numpy_to_data_array
 from .converters import convert_to_dataset, convert_to_inference_data
+from .datasets import clear_data_home, list_datasets, load_arviz_data
+from .inference_data import InferenceData, concat
 from .io_cmdstan import from_cmdstan
 from .io_cmdstanpy import from_cmdstanpy
 from .io_dict import from_dict
+from .io_emcee import from_emcee
+from .io_json import from_json
+from .io_netcdf import from_netcdf, to_netcdf
+from .io_numpyro import from_numpyro
 from .io_pyjags import from_pyjags
 from .io_pymc3 import from_pymc3, from_pymc3_predictions
-from .io_pystan import from_pystan
-from .io_emcee import from_emcee
 from .io_pyro import from_pyro
-from .io_numpyro import from_numpyro
+from .io_pystan import from_pystan
 from .io_tfp import from_tfp
 
 __all__ = [
@@ -33,6 +34,7 @@ __all__ = [
     "from_cmdstan",
     "from_cmdstanpy",
     "from_dict",
+    "from_json",
     "from_pyro",
     "from_numpyro",
     "from_tfp",
