@@ -1,9 +1,10 @@
 """Low level converters usually used by other functions."""
 import datetime
 import functools
+import re
 import warnings
 from copy import deepcopy
-from typing import Any, Dict, List
+from typing import Any, Callable, Dict, List, Optional, Tuple, TypeVar, Union
 
 import numpy as np
 import pkg_resources
@@ -17,9 +18,12 @@ except ImportError:
     import json  # type: ignore
 
 from .. import __version__, utils
+from ..rcparams import rcParams
 
 CoordSpec = Dict[str, List[Any]]
 DimSpec = Dict[str, List[str]]
+RequiresArgTypeT = TypeVar("RequiresArgTypeT")
+RequiresReturnTypeT = TypeVar("RequiresReturnTypeT")
 
 
 class requires:  # pylint: disable=invalid-name
@@ -31,25 +35,46 @@ class requires:  # pylint: disable=invalid-name
     missing. Both functionalities can be combined as desired.
     """
 
-    def __init__(self, *props):
-        self.props = props
+    def __init__(self, *props: Union[str, List[str]]) -> None:
+        self.props: Tuple[Union[str, List[str]], ...] = props
 
-    def __call__(self, func):  # noqa: D202
+    # Until typing.ParamSpec (https://www.python.org/dev/peps/pep-0612/) is available
+    # in all our supported Python versions, there is no way to simultaneously express
+    # the following two properties:
+    # - the input function may take arbitrary args/kwargs, and
+    # - the output function takes those same arbitrary args/kwargs, but has a different return type.
+    # We either have to limit the input function to e.g. only allowing a "self" argument,
+    # or we have to adopt the current approach of annotating the returned function as if
+    # it was defined as "def f(*args: Any, **kwargs: Any) -> Optional[RequiresReturnTypeT]".
+    #
+    # Since all functions decorated with @requires currently only accept a single argument,
+    # we choose to limit application of @requires to only functions of one argument.
+    # When typing.ParamSpec is available, this definition can be updated to use it.
+    # See https://github.com/arviz-devs/arviz/pull/1504 for more discussion.
+    def __call__(
+        self, func: Callable[[RequiresArgTypeT], RequiresReturnTypeT]
+    ) -> Callable[[RequiresArgTypeT], Optional[RequiresReturnTypeT]]:  # noqa: D202
         """Wrap the decorated function."""
 
-        def wrapped(cls, *args, **kwargs):
+        def wrapped(cls: RequiresArgTypeT) -> Optional[RequiresReturnTypeT]:
             """Return None if not all props are available."""
             for prop in self.props:
                 prop = [prop] if isinstance(prop, str) else prop
-                if all([getattr(cls, prop_i) is None for prop_i in prop]):
+                if all((getattr(cls, prop_i) is None for prop_i in prop)):
                     return None
-            return func(cls, *args, **kwargs)
+            return func(cls)
 
         return wrapped
 
 
 def generate_dims_coords(
-    shape, var_name, dims=None, coords=None, default_dims=None, skip_event_dims=None
+    shape,
+    var_name,
+    dims=None,
+    coords=None,
+    default_dims=None,
+    index_origin=None,
+    skip_event_dims=None,
 ):
     """Generate default dimensions and coordinates for a variable.
 
@@ -70,6 +95,9 @@ def generate_dims_coords(
         when manipulating Monte Carlo traces, the ``default_dims`` would be
         ``["chain" , "draw"]`` which ArviZ uses as its own names for dimensions
         of MCMC traces.
+    index_origin : int, optional
+        Starting value of integer coordinate values. Defaults to the value in rcParam
+        ``data.index_origin``.
     skip_event_dims : bool, default False
 
     Returns
@@ -79,6 +107,8 @@ def generate_dims_coords(
     dict[str] -> list[str]
         Default coords
     """
+    if index_origin is None:
+        index_origin = rcParams["data.index_origin"]
     if default_dims is None:
         default_dims = []
     if dims is None:
@@ -113,33 +143,43 @@ def generate_dims_coords(
     if skip_event_dims:
         # this is needed in case the reduction keeps the dimension with size 1
         for i, (dim, dim_size) in enumerate(zip(dims, shape)):
-            print(f"{i}, dim: {dim}, {dim_size} =? {len(coords.get(dim, []))}")
             if (dim in coords) and (dim_size != len(coords[dim])):
                 dims = dims[:i]
                 break
 
     for idx, dim_len in enumerate(shape):
         if (len(dims) < idx + 1) or (dims[idx] is None):
-            dim_name = "{var_name}_dim_{idx}".format(var_name=var_name, idx=idx)
+            dim_name = f"{var_name}_dim_{idx}"
             if len(dims) < idx + 1:
                 dims.append(dim_name)
             else:
                 dims[idx] = dim_name
         dim_name = dims[idx]
         if dim_name not in coords:
-            coords[dim_name] = utils.arange(dim_len)
+            coords[dim_name] = np.arange(index_origin, dim_len + index_origin)
     coords = {key: coord for key, coord in coords.items() if any(key == dim for dim in dims)}
     return dims, coords
 
 
-def numpy_to_data_array(ary, *, var_name="data", coords=None, dims=None, skip_event_dims=None):
+def numpy_to_data_array(
+    ary,
+    *,
+    var_name="data",
+    coords=None,
+    dims=None,
+    default_dims=None,
+    index_origin=None,
+    skip_event_dims=None,
+):
     """Convert a numpy array to an xarray.DataArray.
 
-    The first two dimensions will be (chain, draw), and any remaining
+    By default, the first two dimensions will be (chain, draw), and any remaining
     dimensions will be "shape".
-    If the numpy array is 1d, this dimension is interpreted as draw
-    If the numpy array is 2d, it is interpreted as (chain, draw)
-    If the numpy array is 3 or more dimensions, the last dimensions are kept as shapes.
+    * If the numpy array is 1d, this dimension is interpreted as draw
+    * If the numpy array is 2d, it is interpreted as (chain, draw)
+    * If the numpy array is 3 or more dimensions, the last dimensions are kept as shapes.
+
+    To modify this behaviour, use ``default_dims``.
 
     Parameters
     ----------
@@ -154,6 +194,11 @@ def numpy_to_data_array(ary, *, var_name="data", coords=None, dims=None, skip_ev
         is the name of the dimension, the values are the index values.
     dims : List(str)
         A list of coordinate names for the variable
+    default_dims : list of str, optional
+        Passed to :py:func:`generate_dims_coords`. Defaults to ``["chain", "draw"]``, and
+        an empty list is accepted
+    index_origin : int, optional
+        Passed to :py:func:`generate_dims_coords`
     skip_event_dims : bool
 
     Returns
@@ -162,37 +207,43 @@ def numpy_to_data_array(ary, *, var_name="data", coords=None, dims=None, skip_ev
         Will have the same data as passed, but with coordinates and dimensions
     """
     # manage and transform copies
-    default_dims = ["chain", "draw"]
-    ary = utils.two_de(ary)
-    n_chains, n_samples, *shape = ary.shape
-    if n_chains > n_samples:
-        warnings.warn(
-            "More chains ({n_chains}) than draws ({n_samples}). "
-            "Passed array should have shape (chains, draws, *shape)".format(
-                n_chains=n_chains, n_samples=n_samples
-            ),
-            UserWarning,
-        )
+    if default_dims is None:
+        default_dims = ["chain", "draw"]
+    if "chain" in default_dims and "draw" in default_dims:
+        ary = utils.two_de(ary)
+        n_chains, n_samples, *_ = ary.shape
+        if n_chains > n_samples:
+            warnings.warn(
+                "More chains ({n_chains}) than draws ({n_samples}). "
+                "Passed array should have shape (chains, draws, *shape)".format(
+                    n_chains=n_chains, n_samples=n_samples
+                ),
+                UserWarning,
+            )
+    else:
+        ary = utils.one_de(ary)
 
     dims, coords = generate_dims_coords(
-        shape,
+        ary.shape[len(default_dims) :],
         var_name,
         dims=dims,
         coords=coords,
         default_dims=default_dims,
+        index_origin=index_origin,
         skip_event_dims=skip_event_dims,
     )
 
     # reversed order for default dims: 'chain', 'draw'
-    if "draw" not in dims:
+    if "draw" not in dims and "draw" in default_dims:
         dims = ["draw"] + dims
-    if "chain" not in dims:
+    if "chain" not in dims and "chain" in default_dims:
         dims = ["chain"] + dims
 
-    if "chain" not in coords:
-        coords["chain"] = utils.arange(n_chains)
-    if "draw" not in coords:
-        coords["draw"] = utils.arange(n_samples)
+    index_origin = rcParams["data.index_origin"]
+    if "chain" not in coords and "chain" in default_dims:
+        coords["chain"] = np.arange(index_origin, n_chains + index_origin)
+    if "draw" not in coords and "draw" in default_dims:
+        coords["draw"] = np.arange(index_origin, n_samples + index_origin)
 
     # filter coords based on the dims
     coords = {key: xr.IndexVariable((key,), data=coords[key]) for key in dims}
@@ -200,7 +251,15 @@ def numpy_to_data_array(ary, *, var_name="data", coords=None, dims=None, skip_ev
 
 
 def dict_to_dataset(
-    data, *, attrs=None, library=None, coords=None, dims=None, skip_event_dims=None
+    data,
+    *,
+    attrs=None,
+    library=None,
+    coords=None,
+    dims=None,
+    default_dims=None,
+    index_origin=None,
+    skip_event_dims=None,
 ):
     """Convert a dictionary of numpy arrays to an xarray.Dataset.
 
@@ -217,6 +276,10 @@ def dict_to_dataset(
     dims : dict[str] -> list[str]
         Dimensions of each variable. The keys are variable names, values are lists of
         coordinates.
+    default_dims : list of str, optional
+        Passed to :py:func:`numpy_to_data_array`
+    index_origin : int, optional
+        Passed to :py:func:`numpy_to_data_array`
     skip_event_dims : bool
         If True, cut extra dims whenever present to match the shape of the data.
         Necessary for PPLs which have the same name in both observed data and log
@@ -238,7 +301,13 @@ def dict_to_dataset(
     data_vars = {}
     for key, values in data.items():
         data_vars[key] = numpy_to_data_array(
-            values, var_name=key, coords=coords, dims=dims.get(key), skip_event_dims=skip_event_dims
+            values,
+            var_name=key,
+            coords=coords,
+            dims=dims.get(key),
+            default_dims=default_dims,
+            index_origin=index_origin,
+            skip_event_dims=skip_event_dims,
         )
     return xr.Dataset(data_vars=data_vars, attrs=make_attrs(attrs=attrs, library=library))
 
@@ -312,7 +381,7 @@ def _extend_xr_method(func, doc="", description="", examples="", see_also=""):
         return None if _inplace else out
 
     description_default = """{method_name} method is extended from xarray.Dataset methods.
-    
+
     {description}For more info see :meth:`xarray:xarray.Dataset.{method_name}`
     """.format(
         description=description, method_name=func.__name__  # pylint: disable=no-member
@@ -352,7 +421,7 @@ def _extend_xr_method(func, doc="", description="", examples="", see_also=""):
 
 def _make_json_serializable(data: dict) -> dict:
     """Convert `data` with numpy.ndarray-like values to JSON-serializable form."""
-    ret = dict()
+    ret = {}
     for key, value in data.items():
         try:
             json.dumps(value)
@@ -370,3 +439,47 @@ def _make_json_serializable(data: dict) -> dict:
                 f"Value associated with variable `{type(value)}` is not JSON serializable."
             )
     return ret
+
+
+def infer_stan_dtypes(stan_code):
+    """Infer Stan integer variables from generated quantities block."""
+    # Remove old deprecated comments
+    stan_code = "\n".join(
+        line if "#" not in line else line[: line.find("#")] for line in stan_code.splitlines()
+    )
+    pattern_remove_comments = re.compile(
+        r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"', re.DOTALL | re.MULTILINE
+    )
+    stan_code = re.sub(pattern_remove_comments, "", stan_code)
+
+    # Check generated quantities
+    if "generated quantities" not in stan_code:
+        return {}
+
+    # Extract generated quantities block
+    gen_quantities_location = stan_code.index("generated quantities")
+    block_start = gen_quantities_location + stan_code[gen_quantities_location:].index("{")
+
+    curly_bracket_count = 0
+    block_end = None
+    for block_end, char in enumerate(stan_code[block_start:], block_start + 1):
+        if char == "{":
+            curly_bracket_count += 1
+        elif char == "}":
+            curly_bracket_count -= 1
+
+            if curly_bracket_count == 0:
+                break
+
+    stan_code = stan_code[block_start:block_end]
+
+    stan_integer = r"int"
+    stan_limits = r"(?:\<[^\>]+\>)*"  # ignore group: 0 or more <....>
+    stan_param = r"([^;=\s\[]+)"  # capture group: ends= ";", "=", "[" or whitespace
+    stan_ws = r"\s*"  # 0 or more whitespace
+    stan_ws_one = r"\s+"  # 1 or more whitespace
+    pattern_int = re.compile(
+        "".join((stan_integer, stan_ws_one, stan_limits, stan_ws, stan_param)), re.IGNORECASE
+    )
+    dtypes = {key.strip(): "int" for key in re.findall(pattern_int, stan_code)}
+    return dtypes
