@@ -3,13 +3,12 @@ import logging
 import re
 from collections import defaultdict
 from copy import deepcopy
+from pathlib import Path
 
 import numpy as np
-import xarray as xr
 
-from .. import utils
 from ..rcparams import rcParams
-from .base import dict_to_dataset, generate_dims_coords, make_attrs, requires
+from .base import dict_to_dataset, infer_stan_dtypes, make_attrs, requires
 from .inference_data import InferenceData
 
 _log = logging.getLogger(__name__)
@@ -32,9 +31,11 @@ class CmdStanPyConverter:
         constant_data=None,
         predictions_constant_data=None,
         log_likelihood=None,
+        index_origin=None,
         coords=None,
         dims=None,
         save_warmup=None,
+        dtypes=None,
     ):
         self.posterior = posterior  # CmdStanPy CmdStanMCMC object
         self.posterior_predictive = posterior_predictive
@@ -44,7 +45,10 @@ class CmdStanPyConverter:
         self.observed_data = observed_data
         self.constant_data = constant_data
         self.predictions_constant_data = predictions_constant_data
-        self.log_likelihood = log_likelihood
+        self.log_likelihood = (
+            rcParams["data.log_likelihood"] if log_likelihood is None else log_likelihood
+        )
+        self.index_origin = index_origin
         self.coords = coords
         self.dims = dims
 
@@ -52,6 +56,36 @@ class CmdStanPyConverter:
 
         import cmdstanpy  # pylint: disable=import-error
 
+        if dtypes is None:
+            dtypes = {}
+        elif isinstance(dtypes, cmdstanpy.model.CmdStanModel):
+            model_code = dtypes.code()
+            dtypes = infer_stan_dtypes(model_code)
+        elif isinstance(dtypes, str):
+            dtypes_path = Path(dtypes)
+            if dtypes_path.exists():
+                with dtypes_path.open("r", encoding="UTF-8") as f_obj:
+                    model_code = f_obj.read()
+            else:
+                model_code = dtypes
+            dtypes = infer_stan_dtypes(model_code)
+
+        self.dtypes = dtypes
+
+        if hasattr(self.posterior, "stan_vars_cols"):
+            if self.log_likelihood is True and "log_lik" in self.posterior.stan_vars_cols:
+                self.log_likelihood = ["log_lik"]
+        else:
+            if (
+                self.log_likelihood is True
+                and self.posterior is not None
+                and any(name.split("[")[0] == "log_lik" for name in self.posterior.column_names)
+            ):
+                self.log_likelihood = ["log_lik"]
+
+        if isinstance(self.log_likelihood, bool):
+            self.log_likelihood = None
+
         self.cmdstanpy = cmdstanpy
 
     @requires("posterior")
@@ -85,6 +119,7 @@ class CmdStanPyConverter:
             self.posterior,
             items,
             self.save_warmup,
+            self.dtypes,
         )
 
         # copy dims and coords  - Mitzi question:  why???
@@ -92,8 +127,20 @@ class CmdStanPyConverter:
         coords = deepcopy(self.coords) if self.coords is not None else {}
 
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=coords, dims=dims),
-            dict_to_dataset(data_warmup, library=self.cmdstanpy, coords=coords, dims=dims),
+            dict_to_dataset(
+                data,
+                library=self.cmdstanpy,
+                coords=coords,
+                dims=dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=coords,
+                dims=dims,
+                index_origin=self.index_origin,
+            ),
         )
 
     @requires("posterior")
@@ -111,8 +158,13 @@ class CmdStanPyConverter:
         if not hasattr(fit, "sampler_vars_cols"):
             return self.sample_stats_to_xarray_pre_v_0_9_68(fit)
 
-        dtypes = {"divergent__": bool, "n_leapfrog__": np.int64, "treedepth__": np.int64}
-        items = list(self.posterior.sampler_vars_cols.keys())
+        dtypes = {
+            "divergent__": bool,
+            "n_leapfrog__": np.int64,
+            "treedepth__": np.int64,
+            **self.dtypes,
+        }
+        items = list(fit.sampler_vars_cols.keys())
         rename_dict = {
             "divergent": "diverging",
             "n_leapfrog": "n_steps",
@@ -125,6 +177,7 @@ class CmdStanPyConverter:
             fit,
             items,
             self.save_warmup,
+            self.dtypes,
         )
         for item in items:
             name = re.sub("__$", "", item)
@@ -133,9 +186,19 @@ class CmdStanPyConverter:
             if data_warmup:
                 data_warmup[name] = data_warmup.pop(item).astype(dtypes.get(item, float))
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=self.coords, dims=self.dims),
             dict_to_dataset(
-                data_warmup, library=self.cmdstanpy, coords=self.coords, dims=self.dims
+                data,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
             ),
         )
 
@@ -160,6 +223,7 @@ class CmdStanPyConverter:
                 fit,
                 predictive,
                 self.save_warmup,
+                self.dtypes,
             )
         else:  # pre_v_0_9_68
             valid_cols = _filter_columns(fit.column_names, predictive)
@@ -168,12 +232,23 @@ class CmdStanPyConverter:
                 fit.column_names,
                 valid_cols,
                 self.save_warmup,
+                self.dtypes,
             )
 
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=self.coords, dims=self.dims),
             dict_to_dataset(
-                data_warmup, library=self.cmdstanpy, coords=self.coords, dims=self.dims
+                data,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
             ),
         )
 
@@ -188,6 +263,7 @@ class CmdStanPyConverter:
                 self.posterior,
                 predictions,
                 self.save_warmup,
+                self.dtypes,
             )
         else:  # pre_v_0_9_68
             columns = self.posterior.column_names
@@ -197,12 +273,23 @@ class CmdStanPyConverter:
                 columns,
                 valid_cols,
                 self.save_warmup,
+                self.dtypes,
             )
 
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=self.coords, dims=self.dims),
             dict_to_dataset(
-                data_warmup, library=self.cmdstanpy, coords=self.coords, dims=self.dims
+                data,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
             ),
         )
 
@@ -217,6 +304,7 @@ class CmdStanPyConverter:
                 self.posterior,
                 log_likelihood,
                 self.save_warmup,
+                self.dtypes,
             )
         else:  # pre_v_0_9_68
             columns = self.posterior.column_names
@@ -226,13 +314,22 @@ class CmdStanPyConverter:
                 columns,
                 valid_cols,
                 self.save_warmup,
+                self.dtypes,
             )
+        if isinstance(self.log_likelihood, dict):
+            data = {obs_name: data[lik_name] for obs_name, lik_name in self.log_likelihood.items()}
+            if data_warmup:
+                data_warmup = {
+                    obs_name: data_warmup[lik_name]
+                    for obs_name, lik_name in self.log_likelihood.items()
+                }
         return (
             dict_to_dataset(
                 data,
                 library=self.cmdstanpy,
                 coords=self.coords,
                 dims=self.dims,
+                index_origin=self.index_origin,
                 skip_event_dims=True,
             ),
             dict_to_dataset(
@@ -240,6 +337,7 @@ class CmdStanPyConverter:
                 library=self.cmdstanpy,
                 coords=self.coords,
                 dims=self.dims,
+                index_origin=self.index_origin,
                 skip_event_dims=True,
             ),
         )
@@ -247,17 +345,18 @@ class CmdStanPyConverter:
     @requires("prior")
     def prior_to_xarray(self):
         """Convert prior samples to xarray."""
-        if hasattr(self.posterior, "stan_vars_cols"):
-            items = list(self.posterior.stan_vars_cols.keys())
+        if hasattr(self.prior, "stan_vars_cols"):
+            items = list(self.prior.stan_vars_cols.keys())
             if self.prior_predictive is not None:
                 try:
                     items = _filter(items, self.prior_predictive)
                 except ValueError:
                     pass
             data, data_warmup = _unpack_fit(
-                self.posterior,
+                self.prior,
                 items,
                 self.save_warmup,
+                self.dtypes,
             )
         else:  # pre_v_0_9_68
             columns = self.prior.column_names
@@ -272,54 +371,61 @@ class CmdStanPyConverter:
                 columns,
                 valid_cols,
                 self.save_warmup,
+                self.dtypes,
             )
 
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=self.coords, dims=self.dims),
             dict_to_dataset(
-                data_warmup, library=self.cmdstanpy, coords=self.coords, dims=self.dims
+                data,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
             ),
         )
 
     @requires("observed_data")
     def observed_data_to_xarray(self):
         """Convert observed data to xarray."""
-        observed_data = {}
-        for key, vals in self.observed_data.items():
-            vals = utils.one_de(vals)
-            val_dims = self.dims.get(key) if self.dims is not None else None
-            val_dims, coords = generate_dims_coords(
-                vals.shape, key, dims=val_dims, coords=self.coords
-            )
-            observed_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
-        return xr.Dataset(data_vars=observed_data, attrs=make_attrs(library=self.cmdstanpy))
+        return dict_to_dataset(
+            self.observed_data,
+            library=self.cmdstanpy,
+            coords=self.coords,
+            dims=self.dims,
+            default_dims=[],
+            index_origin=self.index_origin,
+        )
 
     @requires("constant_data")
     def constant_data_to_xarray(self):
         """Convert constant data to xarray."""
-        constant_data = {}
-        for key, vals in self.constant_data.items():
-            vals = utils.one_de(vals)
-            val_dims = self.dims.get(key) if self.dims is not None else None
-            val_dims, coords = generate_dims_coords(
-                vals.shape, key, dims=val_dims, coords=self.coords
-            )
-            constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
-        return xr.Dataset(data_vars=constant_data, attrs=make_attrs(library=self.cmdstanpy))
+        return dict_to_dataset(
+            self.constant_data,
+            library=self.cmdstanpy,
+            coords=self.coords,
+            dims=self.dims,
+            default_dims=[],
+            index_origin=self.index_origin,
+        )
 
     @requires("predictions_constant_data")
     def predictions_constant_data_to_xarray(self):
         """Convert constant data to xarray."""
-        predictions_constant_data = {}
-        for key, vals in self.predictions_constant_data.items():
-            vals = utils.one_de(vals)
-            val_dims = self.dims.get(key) if self.dims is not None else None
-            val_dims, coords = generate_dims_coords(
-                vals.shape, key, dims=val_dims, coords=self.coords
-            )
-            predictions_constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
-        return xr.Dataset(
-            data_vars=predictions_constant_data, attrs=make_attrs(library=self.cmdstanpy)
+        return dict_to_dataset(
+            self.predictions_constant_data,
+            library=self.cmdstanpy,
+            coords=self.coords,
+            dims=self.dims,
+            attrs=make_attrs(library=self.cmdstanpy),
+            default_dims=[],
+            index_origin=self.index_origin,
         )
 
     def to_inference_data(self):
@@ -404,16 +510,26 @@ class CmdStanPyConverter:
             columns,
             valid_cols,
             self.save_warmup,
+            self.dtypes,
         )
 
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=self.coords, dims=self.dims),
             dict_to_dataset(
-                data_warmup, library=self.cmdstanpy, coords=self.coords, dims=self.dims
+                data,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
             ),
         )
 
-    @requires("posterior")
     def sample_stats_to_xarray_pre_v_0_9_68(self, fit):
         """Extract sample_stats from fit."""
         dtypes = {"divergent__": bool, "n_leapfrog__": np.int64, "treedepth__": np.int64}
@@ -424,6 +540,7 @@ class CmdStanPyConverter:
             columns,
             valid_cols,
             self.save_warmup,
+            self.dtypes,
         )
         for s_param in list(data.keys()):
             s_param_, *_ = s_param.split(".")
@@ -433,9 +550,19 @@ class CmdStanPyConverter:
             if data_warmup:
                 data_warmup[name] = data_warmup.pop(s_param).astype(dtypes.get(s_param, float))
         return (
-            dict_to_dataset(data, library=self.cmdstanpy, coords=self.coords, dims=self.dims),
             dict_to_dataset(
-                data_warmup, library=self.cmdstanpy, coords=self.coords, dims=self.dims
+                data,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                library=self.cmdstanpy,
+                coords=self.coords,
+                dims=self.dims,
+                index_origin=self.index_origin,
             ),
         )
 
@@ -446,7 +573,9 @@ def _as_set(spec):
         return []
     if isinstance(spec, str):
         return [spec]
-    else:
+    try:
+        return set(spec.values())
+    except AttributeError:
         return set(spec)
 
 
@@ -458,7 +587,7 @@ def _filter(names, spec):
         for item in spec:
             names.remove(item)
     elif isinstance(spec, dict):
-        for item in spec.keys():
+        for item in spec.values():
             names.remove(item)
     return names
 
@@ -468,7 +597,7 @@ def _filter_columns(columns, spec):
     return [col for col in columns if col.split("[")[0].split(".")[0] in spec]
 
 
-def _unpack_fit(fit, items, save_warmup):
+def _unpack_fit(fit, items, save_warmup, dtypes):
     """Transform fit to dictionary containing ndarrays.
 
     Parameters
@@ -476,6 +605,7 @@ def _unpack_fit(fit, items, save_warmup):
     data: cmdstanpy.CmdStanMCMC
     items: list
     save_warmup: bool
+    dtypes: dict
 
     Returns
     -------
@@ -489,6 +619,7 @@ def _unpack_fit(fit, items, save_warmup):
         else:
             num_warmup = fit.num_draws_warmup
 
+    nchains = fit.chains
     draws = np.swapaxes(fit.draws(inc_warmup=save_warmup), 0, 1)
     sample = {}
     sample_warmup = {}
@@ -496,27 +627,29 @@ def _unpack_fit(fit, items, save_warmup):
     for item in items:
         if item in fit.stan_vars_cols:
             col_idxs = fit.stan_vars_cols[item]
+            if len(col_idxs) == 1:
+                raw_draws = draws[..., col_idxs[0]]
+            else:
+                raw_draws = fit.stan_variable(item, inc_warmup=save_warmup)
+                raw_draws = np.swapaxes(
+                    raw_draws.reshape((-1, nchains, *raw_draws.shape[1:]), order="F"), 0, 1
+                )
         elif item in fit.sampler_vars_cols:
             col_idxs = fit.sampler_vars_cols[item]
+            raw_draws = draws[..., col_idxs[0]]
         else:
-            raise ValueError("fit data, unknown variable: {}".format(item))
+            raise ValueError(f"fit data, unknown variable: {item}")
+        raw_draws = raw_draws.astype(dtypes.get(item))
         if save_warmup:
-            if len(col_idxs) == 1:
-                sample_warmup[item] = np.squeeze(draws[:num_warmup, :, col_idxs], axis=2)
-                sample[item] = np.squeeze(draws[num_warmup:, :, col_idxs], axis=2)
-            else:
-                sample_warmup[item] = draws[:num_warmup, :, col_idxs]
-                sample[item] = draws[num_warmup:, :, col_idxs]
+            sample_warmup[item] = raw_draws[:, :num_warmup, ...]
+            sample[item] = raw_draws[:, num_warmup:, ...]
         else:
-            if len(col_idxs) == 1:
-                sample[item] = np.squeeze(draws[:, :, col_idxs], axis=2)
-            else:
-                sample[item] = draws[:, :, col_idxs]
+            sample[item] = raw_draws
 
     return sample, sample_warmup
 
 
-def _unpack_frame(fit, columns, valid_cols, save_warmup):
+def _unpack_frame(fit, columns, valid_cols, save_warmup, dtypes):
     """Transform fit to dictionary containing ndarrays.
 
     Called when fit object created by cmdstanpy version < 0.9.68
@@ -527,6 +660,7 @@ def _unpack_frame(fit, columns, valid_cols, save_warmup):
     columns: list
     valid_cols: list
     save_warmup: bool
+    dtypes: dict
 
     Returns
     -------
@@ -600,6 +734,13 @@ def _unpack_frame(fit, columns, valid_cols, save_warmup):
                 sample[key][shape_loc] = np.swapaxes(data[..., i], 0, 1)
                 if save_warmup:
                     sample_warmup[key][shape_loc] = np.swapaxes(data_warmup[..., i], 0, 1)
+
+    for key, dtype in dtypes.items():
+        if key in sample:
+            sample[key] = sample[key].astype(dtype)
+            if save_warmup:
+                if key in sample_warmup:
+                    sample_warmup[key] = sample_warmup[key].astype(dtype)
     return sample, sample_warmup
 
 
@@ -614,9 +755,11 @@ def from_cmdstanpy(
     constant_data=None,
     predictions_constant_data=None,
     log_likelihood=None,
+    index_origin=None,
     coords=None,
     dims=None,
     save_warmup=None,
+    dtypes=None,
 ):
     """Convert CmdStanPy data into an InferenceData object.
 
@@ -641,8 +784,15 @@ def from_cmdstanpy(
         Constant data used in the sampling.
     predictions_constant_data : dict
         Constant data for predictions used in the sampling.
-    log_likelihood : str, list of str
-        Pointwise log_likelihood for the data.
+    log_likelihood : str, list of str, dict of {str: str}, optional
+        Pointwise log_likelihood for the data. If a dict, its keys should represent var_names
+        from the corresponding observed data and its values the stan variable where the
+        data is stored. By default, if a variable ``log_lik`` is present in the Stan model,
+        it will be retrieved as pointwise log likelihood values. Use ``False``
+        or set ``data.log_likelihood`` to false to avoid this behaviour.
+    index_origin : int, optional
+        Starting value of integer coordinate values. Defaults to the value in rcParam
+        ``data.index_origin``.
     coords : dict of str or dict of iterable
         A dictionary containing the values that are used as index. The key
         is the name of the dimension, the values are the index values.
@@ -651,6 +801,10 @@ def from_cmdstanpy(
     save_warmup : bool
         Save warmup iterations into InferenceData object, if found in the input files.
         If not defined, use default defined by the rcParams.
+    dtypes: dict or str or cmdstanpy.CmdStanModel
+        A dictionary containing dtype information (int, float) for parameters.
+        If input is a string, it is assumed to be a model code or path to model code file.
+        Model code can extracted from cmdstanpy.CmdStanModel object.
 
     Returns
     -------
@@ -666,7 +820,9 @@ def from_cmdstanpy(
         constant_data=constant_data,
         predictions_constant_data=predictions_constant_data,
         log_likelihood=log_likelihood,
+        index_origin=index_origin,
         coords=coords,
         dims=dims,
         save_warmup=save_warmup,
+        dtypes=dtypes,
     ).to_inference_data()
