@@ -2,7 +2,8 @@
 import matplotlib.pyplot as plt
 import numpy as np
 
-from . import backend_kwarg_defaults, backend_show
+from ...plot_utils import _scale_fig_size
+from . import backend_kwarg_defaults, backend_show, create_axes_grid
 
 
 def plot_parallel(
@@ -11,13 +12,13 @@ def plot_parallel(
     colord,
     shadend,
     diverging_mask,
-    _posterior,
+    posterior,
     textsize,
     var_names,
-    xt_labelsize,
     legend,
     figsize,
     backend_kwargs,
+    backend_config,  # pylint: disable=unused-argument
     show,
 ):
     """Matplotlib parallel plot."""
@@ -28,13 +29,17 @@ def plot_parallel(
         **backend_kwarg_defaults(),
         **backend_kwargs,
     }
-    if ax is None:
-        _, ax = plt.subplots(figsize=figsize, **backend_kwargs)
 
-    ax.plot(_posterior[:, ~diverging_mask], color=colornd, alpha=shadend)
+    figsize, _, _, xt_labelsize, _, _ = _scale_fig_size(figsize, textsize, 1, 1)
+    backend_kwargs.setdefault("figsize", figsize)
+    backend_kwargs["squeeze"] = True
+    if ax is None:
+        _, ax = create_axes_grid(1, backend_kwargs=backend_kwargs)
+
+    ax.plot(posterior[:, ~diverging_mask], color=colornd, alpha=shadend)
 
     if np.any(diverging_mask):
-        ax.plot(_posterior[:, diverging_mask], color=colord, lw=1)
+        ax.plot(posterior[:, diverging_mask], color=colord, lw=1)
 
     ax.tick_params(labelsize=textsize)
     ax.set_xticks(range(len(var_names)))
