@@ -11,8 +11,8 @@ from ...plot_utils import (
     _scale_fig_size,
     calculate_point_estimate,
     format_sig_figs,
-    make_label,
     round_num,
+    vectorized_to_hex,
 )
 from . import backend_kwarg_defaults, backend_show, create_axes_grid, matplotlib_kwarg_dealiaser
 
@@ -36,6 +36,9 @@ def plot_posterior(
     textsize,
     ref_val,
     rope,
+    ref_val_color,
+    rope_color,
+    labeller,
     kwargs,
     backend_kwargs,
     show,
@@ -69,7 +72,7 @@ def plot_posterior(
             backend_kwargs=backend_kwargs,
         )
     idx = 0
-    for (var_name, selection, x), ax_ in zip(plotters, np.ravel(ax)):
+    for (var_name, selection, isel, x), ax_ in zip(plotters, np.ravel(ax)):
         _plot_posterior_op(
             idx,
             x.flatten(),
@@ -87,12 +90,16 @@ def plot_posterior(
             skipna=skipna,
             ref_val=ref_val,
             rope=rope,
+            ref_val_color=ref_val_color,
+            rope_color=rope_color,
             ax_labelsize=ax_labelsize,
             xt_labelsize=xt_labelsize,
             **kwargs,
         )
         idx += 1
-        ax_.set_title(make_label(var_name, selection), fontsize=titlesize, wrap=True)
+        ax_.set_title(
+            labeller.make_label_vert(var_name, selection, isel), fontsize=titlesize, wrap=True
+        )
 
     if backend_show(show):
         plt.show()
@@ -117,6 +124,8 @@ def _plot_posterior_op(
     skipna,
     ref_val,
     rope,
+    ref_val_color,
+    rope_color,
     ax_labelsize,
     xt_labelsize,
     round_to=None,
@@ -156,15 +165,22 @@ def _plot_posterior_op(
             val,
             format_as_percent(greater_than_ref_probability, 1),
         )
-        ax.axvline(val, ymin=0.05, ymax=0.75, color="C1", lw=linewidth, alpha=0.65)
+        ax.axvline(
+            val,
+            ymin=0.05,
+            ymax=0.75,
+            lw=linewidth,
+            alpha=0.65,
+            color=vectorized_to_hex(ref_val_color),
+        )
         ax.text(
             values.mean(),
             plot_height * 0.6,
             ref_in_posterior,
             size=ax_labelsize,
-            color="C1",
             weight="semibold",
             horizontalalignment="center",
+            color=vectorized_to_hex(ref_val_color),
         )
 
     def display_rope():
@@ -188,24 +204,33 @@ def _plot_posterior_op(
                 "iterable of length 2"
             )
         rope_text = [f"{val:.{format_sig_figs(val, round_to)}g}" for val in vals]
-
         ax.plot(
             vals,
             (plot_height * 0.02, plot_height * 0.02),
             lw=linewidth * 5,
-            color="C2",
             solid_capstyle="butt",
             zorder=0,
             alpha=0.7,
+            color=vectorized_to_hex(rope_color),
+        )
+        probability_within_rope = ((values > vals[0]) & (values <= vals[1])).mean()
+        ax.text(
+            values.mean(),
+            plot_height * 0.45,
+            f"{format_as_percent(probability_within_rope, 1)} in ROPE",
+            weight="semibold",
+            horizontalalignment="center",
+            size=ax_labelsize,
+            color=vectorized_to_hex(rope_color),
         )
-        text_props = {"size": ax_labelsize, "color": "C2"}
         ax.text(
             vals[0],
             plot_height * 0.2,
             rope_text[0],
             weight="semibold",
             horizontalalignment="right",
-            **text_props,
+            size=ax_labelsize,
+            color=vectorized_to_hex(rope_color),
         )
         ax.text(
             vals[1],
@@ -213,7 +238,8 @@ def _plot_posterior_op(
             rope_text[1],
             weight="semibold",
             horizontalalignment="left",
-            **text_props,
+            size=ax_labelsize,
+            color=vectorized_to_hex(rope_color),
         )
 
     def display_point_estimate():
@@ -286,7 +312,7 @@ def _plot_posterior_op(
         plot_kde(
             values,
             bw=bw,
-            circular=circular,
+            is_circular=circular,
             fill_kwargs={"alpha": kwargs.pop("fill_alpha", 0)},
             plot_kwargs=kwargs,
             ax=ax,
