@@ -40,14 +40,15 @@ def plot_khat(
 
     Parameters
     ----------
-    khats : ELPDData cointaining Pareto shapes information or array of
+    khats : ELPDData containing Pareto shapes information or array of
         Pareto tail indices.
     color : str or array_like, optional
         Colors of the scatter plot, if color is a str all dots will
         have the same color, if it is the size of the observations,
         each dot will have the specified color, otherwise, it will be
         interpreted as a list of the dims to be used for the color
-        code
+        code. If Matplotlib c argument is passed, it will override
+        the color argument
     xlabels : bool, optional
         Use coords as xticklabels
     show_hlines : bool, optional
@@ -143,7 +144,7 @@ def plot_khat(
 
     References
     ----------
-    ..[1] Vehtari, A., Simpson, D., Gelman, A., Yao, Y., Gabry, J.,
+    .. [1] Vehtari, A., Simpson, D., Gelman, A., Yao, Y., Gabry, J.,
     2019. Pareto Smoothed Importance Sampling. arXiv:1507.02646
     [stat].
     """
