@@ -1,13 +1,11 @@
 """Utilities for plotting."""
 import importlib
 import warnings
-from itertools import product, tee
 from typing import Any, Dict
 
 import matplotlib as mpl
 import numpy as np
 import packaging
-import xarray as xr
 from matplotlib.colors import to_hex
 from scipy.stats import mode, rankdata
 from scipy.interpolate import CubicSpline
@@ -15,6 +13,7 @@ from scipy.interpolate import CubicSpline
 
 from ..rcparams import rcParams
 from ..stats.density_utils import kde
+from ..stats import hdi
 
 KwargSpec = Dict[str, Any]
 
@@ -143,51 +142,6 @@ def default_grid(n_items, grid=None, max_cols=4, min_cols=3):  # noqa: D202
         return rows, cols
 
 
-def selection_to_string(selection):
-    """Convert dictionary of coordinates to a string for labels.
-
-    Parameters
-    ----------
-    selection : dict[Any] -> Any
-
-    Returns
-    -------
-    str
-        key1: value1, key2: value2, ...
-    """
-    return ", ".join(["{}".format(v) for _, v in selection.items()])
-
-
-def make_label(var_name, selection, position="below"):
-    """Consistent labelling for plots.
-
-    Parameters
-    ----------
-    var_name : str
-       Name of the variable
-
-    selection : dict[Any] -> Any
-        Coordinates of the variable
-    position : str
-        Whether to position the coordinates' label "below" (default) or "beside"
-        the name of the variable
-
-    Returns
-    -------
-    label
-        A text representation of the label
-    """
-    if selection:
-        sel = selection_to_string(selection)
-        if position == "below":
-            sep = "\n"
-        elif position == "beside":
-            sep = " "
-    else:
-        sep = sel = ""
-    return "{}{}{}".format(var_name, sep, sel)
-
-
 def format_sig_figs(value, default=None):
     """Get a default number of significant figures.
 
@@ -222,179 +176,6 @@ def round_num(n, round_to):
     return "{n:.{sig_figs}g}".format(n=n, sig_figs=sig_figs)
 
 
-def purge_duplicates(list_in):
-    """Remove duplicates from list while preserving order.
-
-    Parameters
-    ----------
-    list_in: Iterable
-
-    Returns
-    -------
-    list
-        List of first occurrences in order
-    """
-    # Algorithm taken from Stack Overflow,
-    # https://stackoverflow.com/questions/480214. Content by Georgy
-    # Skorobogatov (https://stackoverflow.com/users/7851470/georgy) and
-    # Markus Jarderot
-    # (https://stackoverflow.com/users/22364/markus-jarderot), licensed
-    # under CC-BY-SA 4.0.
-    # https://creativecommons.org/licenses/by-sa/4.0/.
-
-    seen = set()
-    seen_add = seen.add
-    return [x for x in list_in if not (x in seen or seen_add(x))]
-
-
-def _dims(data, var_name, skip_dims):
-    return [dim for dim in data[var_name].dims if dim not in skip_dims]
-
-
-def _zip_dims(new_dims, vals):
-    return [{k: v for k, v in zip(new_dims, prod)} for prod in product(*vals)]
-
-
-def xarray_sel_iter(data, var_names=None, combined=False, skip_dims=None, reverse_selections=False):
-    """Convert xarray data to an iterator over variable names and selections.
-
-    Iterates over each var_name and all of its coordinates, returning the variable
-    names and selections that allow properly obtain the data from ``data`` as desired.
-
-    Parameters
-    ----------
-    data : xarray.Dataset
-        Posterior data in an xarray
-
-    var_names : iterator of strings (optional)
-        Should be a subset of data.data_vars. Defaults to all of them.
-
-    combined : bool
-        Whether to combine chains or leave them separate
-
-    skip_dims : set
-        dimensions to not iterate over
-
-    reverse_selections : bool
-        Whether to reverse selections before iterating.
-
-    Returns
-    -------
-    Iterator of (var_name: str, selection: dict(str, any))
-        The string is the variable name, the dictionary are coordinate names to values,.
-        To get the values of the variable at these coordinates, do
-        ``data[var_name].sel(**selection)``.
-    """
-    if skip_dims is None:
-        skip_dims = set()
-
-    if combined:
-        skip_dims = skip_dims.union({"chain", "draw"})
-    else:
-        skip_dims.add("draw")
-
-    if var_names is None:
-        if isinstance(data, xr.Dataset):
-            var_names = list(data.data_vars)
-        elif isinstance(data, xr.DataArray):
-            var_names = [data.name]
-            data = {data.name: data}
-
-    for var_name in var_names:
-        if var_name in data:
-            new_dims = _dims(data, var_name, skip_dims)
-            vals = [purge_duplicates(data[var_name][dim].values) for dim in new_dims]
-            dims = _zip_dims(new_dims, vals)
-            if reverse_selections:
-                dims = reversed(dims)
-
-            for selection in dims:
-                yield var_name, selection
-
-
-def xarray_var_iter(data, var_names=None, combined=False, skip_dims=None, reverse_selections=False):
-    """Convert xarray data to an iterator over vectors.
-
-    Iterates over each var_name and all of its coordinates, returning the 1d
-    data.
-
-    Parameters
-    ----------
-    data : xarray.Dataset
-        Posterior data in an xarray
-
-    var_names : iterator of strings (optional)
-        Should be a subset of data.data_vars. Defaults to all of them.
-
-    combined : bool
-        Whether to combine chains or leave them separate
-
-    skip_dims : set
-        dimensions to not iterate over
-
-    reverse_selections : bool
-        Whether to reverse selections before iterating.
-
-    Returns
-    -------
-    Iterator of (str, dict(str, any), np.array)
-        The string is the variable name, the dictionary are coordinate names to values,
-        and the array are the values of the variable at those coordinates.
-    """
-    data_to_sel = data
-    if var_names is None and isinstance(data, xr.DataArray):
-        data_to_sel = {data.name: data}
-
-    for var_name, selection in xarray_sel_iter(
-        data,
-        var_names=var_names,
-        combined=combined,
-        skip_dims=skip_dims,
-        reverse_selections=reverse_selections,
-    ):
-        yield var_name, selection, data_to_sel[var_name].sel(**selection).values
-
-
-def xarray_to_ndarray(data, *, var_names=None, combined=True):
-    """Take xarray data and unpacks into variables and data into list and numpy array respectively.
-
-    Assumes that chain and draw are in coordinates
-
-    Parameters
-    ----------
-    data: xarray.DataSet
-        Data in an xarray from an InferenceData object. Examples include posterior or sample_stats
-
-    var_names: iter
-        Should be a subset of data.data_vars not including chain and draws. Defaults to all of them
-
-    combined: bool
-        Whether to combine chain into one array
-
-    Returns
-    -------
-    var_names: list
-        List of variable names
-    data: np.array
-        Data values
-    """
-    data_to_sel = data
-    if var_names is None and isinstance(data, xr.DataArray):
-        data_to_sel = {data.name: data}
-
-    iterator1, iterator2 = tee(xarray_sel_iter(data, var_names=var_names, combined=combined))
-    vars_and_sel = list(iterator1)
-    unpacked_var_names = [make_label(var_name, selection) for var_name, selection in vars_and_sel]
-
-    # Merge chains and variables, check dtype to be compatible with divergences data
-    data0 = data_to_sel[vars_and_sel[0][0]].sel(**vars_and_sel[0][1])
-    unpacked_data = np.empty((len(unpacked_var_names), data0.size), dtype=data0.dtype)
-    for idx, (var_name, selection) in enumerate(iterator2):
-        unpacked_data[idx] = data_to_sel[var_name].sel(**selection).values.flatten()
-
-    return unpacked_var_names, unpacked_data
-
-
 def color_from_dim(dataarray, dim_name):
     """Return colors and color mapping of a DataArray using coord values as color code.
 
@@ -465,7 +246,7 @@ def format_coords_as_labels(dataarray, skip_dims=None):
         fmt = ", ".join(["{}" for _ in coord_labels[0]])
         coord_labels[:] = [fmt.format(*x) for x in coord_labels]
     else:
-        coord_labels[:] = ["{}".format(s) for s in coord_labels]
+        coord_labels[:] = [f"{s}" for s in coord_labels]
     return coord_labels
 
 
@@ -483,7 +264,7 @@ def set_xticklabels(ax, coord_labels):
 
 
 def filter_plotters_list(plotters, plot_kind):
-    """Cut list of plotters so that it is at most of lenght "plot.max_subplots"."""
+    """Cut list of plotters so that it is at most of length "plot.max_subplots"."""
     max_plots = rcParams["plot.max_subplots"]
     max_plots = len(plotters) if max_plots is None else max_plots
     if len(plotters) > max_plots:
@@ -533,11 +314,7 @@ def get_plotting_function(plot_name, plot_module, backend):
 
     # Perform import of plotting method
     # TODO: Convert module import to top level for all plots
-    module = importlib.import_module(
-        "arviz.plots.backends.{backend}.{plot_module}".format(
-            backend=backend, plot_module=plot_module
-        )
-    )
+    module = importlib.import_module(f"arviz.plots.backends.{backend}.{plot_module}")
 
     plotting_method = getattr(module, plot_name)
 
@@ -605,6 +382,138 @@ def calculate_point_estimate(point_estimate, values, bw="default", circular=Fals
     return point_value
 
 
+def plot_point_interval(
+    ax,
+    values,
+    point_estimate,
+    hdi_prob,
+    quartiles,
+    linewidth,
+    markersize,
+    markercolor,
+    marker,
+    rotated,
+    intervalcolor,
+    backend="matplotlib",
+):
+    """Plot point intervals.
+
+    Translates the data and represents them as point and interval summaries.
+
+    Parameters
+    ----------
+    ax : axes
+        Matplotlib axes
+    values : array-like
+        Values to plot
+    point_estimate : str
+        Plot point estimate per variable.
+    linewidth : int
+        Line width throughout.
+    quartiles : bool
+        If True then the quartile interval will be plotted with the HDI.
+    markersize : int
+        Markersize throughout.
+    markercolor: string
+        Color of the marker.
+    marker: string
+        Shape of the marker.
+    hdi_prob : float
+        Valid only when point_interval is True. Plots HDI for chosen percentage of density.
+    rotated : bool
+        Whether to rotate the dot plot by 90 degrees.
+    intervalcolor : string
+        Color of the interval.
+    backend : string, optional
+        Matplotlib or Bokeh.
+    """
+    endpoint = (1 - hdi_prob) / 2
+    if quartiles:
+        qlist_interval = [endpoint, 0.25, 0.75, 1 - endpoint]
+    else:
+        qlist_interval = [endpoint, 1 - endpoint]
+    quantiles_interval = np.quantile(values, qlist_interval)
+
+    quantiles_interval[0], quantiles_interval[-1] = hdi(
+        values.flatten(), hdi_prob, multimodal=False
+    )
+    mid = len(quantiles_interval) // 2
+    param_iter = zip(np.linspace(2 * linewidth, linewidth, mid, endpoint=True)[-1::-1], range(mid))
+
+    if backend == "matplotlib":
+        for width, j in param_iter:
+            if rotated:
+                ax.vlines(
+                    0,
+                    quantiles_interval[j],
+                    quantiles_interval[-(j + 1)],
+                    linewidth=width,
+                    color=intervalcolor,
+                )
+            else:
+                ax.hlines(
+                    0,
+                    quantiles_interval[j],
+                    quantiles_interval[-(j + 1)],
+                    linewidth=width,
+                    color=intervalcolor,
+                )
+
+        if point_estimate:
+            point_value = calculate_point_estimate(point_estimate, values)
+            if rotated:
+                ax.plot(
+                    0,
+                    point_value,
+                    marker,
+                    markersize=markersize,
+                    color=markercolor,
+                )
+            else:
+                ax.plot(
+                    point_value,
+                    0,
+                    marker,
+                    markersize=markersize,
+                    color=markercolor,
+                )
+    else:
+        for width, j in param_iter:
+            if rotated:
+                ax.line(
+                    [0, 0],
+                    [quantiles_interval[j], quantiles_interval[-(j + 1)]],
+                    line_width=width,
+                    color=intervalcolor,
+                )
+            else:
+                ax.line(
+                    [quantiles_interval[j], quantiles_interval[-(j + 1)]],
+                    [0, 0],
+                    line_width=width,
+                    color=intervalcolor,
+                )
+
+        if point_estimate:
+            point_value = calculate_point_estimate(point_estimate, values)
+            if rotated:
+                ax.circle(
+                    x=0,
+                    y=point_value,
+                    size=markersize,
+                    fill_color=markercolor,
+                )
+            else:
+                ax.circle(
+                    x=point_value,
+                    y=0,
+                    size=markersize,
+                    fill_color=markercolor,
+                )
+
+    return ax
+
+
 def is_valid_quantile(value):
     """Check if value is a number between 0 and 1."""
     try:
@@ -675,7 +584,7 @@ def set_bokeh_circular_ticks_labels(ax, hist, labels):
 
 
 def compute_ranks(ary):
-    """Compute ranks for continuos and discrete variables."""
+    """Compute ranks for continuous and discrete variables."""
     if ary.dtype.kind == "i":
         ary_shape = ary.shape
         ary = ary.flatten()
