@@ -1,12 +1,15 @@
 """Posterior/Prior predictive plot."""
 import logging
+import warnings
 from numbers import Integral
 
 import numpy as np
 
+from ..labels import BaseLabeller
+from ..sel_utils import xarray_var_iter
 from ..rcparams import rcParams
 from ..utils import _var_names
-from .plot_utils import default_grid, filter_plotters_list, get_plotting_function, xarray_var_iter
+from .plot_utils import default_grid, filter_plotters_list, get_plotting_function
 
 _log = logging.getLogger(__name__)
 
@@ -17,7 +20,8 @@ def plot_ppc(
     alpha=None,
     mean=True,
     observed=True,
-    color="C0",
+    color=None,
+    colors=None,
     grid=None,
     figsize=None,
     textsize=None,
@@ -33,6 +37,7 @@ def plot_ppc(
     animated=False,
     animation_kwargs=None,
     legend=True,
+    labeller=None,
     ax=None,
     backend=None,
     backend_kwargs=None,
@@ -57,6 +62,10 @@ def plot_ppc(
         Whether or not to plot the observed data.
     color: str
         Valid matplotlib color. Defaults to C0
+    color: list
+        List with valid matplotlib colors corresponding to the posterior/prior predictive
+        distribution, observed data and mean of the posterior/prior predictive distribution.
+        Defaults to ["C0", "k", "C1"]
     grid : tuple
         Number of rows and columns. Defaults to None, the rows and columns are
         automatically inferred.
@@ -123,6 +132,9 @@ def plot_ppc(
         Keywords passed to `animation.FuncAnimation`. Ignored with matploblib backend.
     legend : bool
         Add legend to figure. By default True.
+    labeller : labeller instance, optional
+        Class providing the method `make_pp_label` to generate the labels in the plot titles.
+        Read the :ref:`label_guide` for more details and usage examples.
     ax: numpy array-like of matplotlib axes or bokeh figures, optional
         A 2D array of locations into which to plot the densities. If not supplied, Arviz will create
         its own array of plot areas (and return it).
@@ -193,15 +205,26 @@ def plot_ppc(
     if group not in ("posterior", "prior"):
         raise TypeError("`group` argument must be either `posterior` or `prior`")
 
-    for groups in ("{}_predictive".format(group), "observed_data"):
+    for groups in (f"{group}_predictive", "observed_data"):
         if not hasattr(data, groups):
-            raise TypeError(
-                '`data` argument must have the group "{group}" for ppcplot'.format(group=groups)
-            )
+            raise TypeError(f'`data` argument must have the group "{groups}" for ppcplot')
 
     if kind.lower() not in ("kde", "cumulative", "scatter"):
         raise TypeError("`kind` argument must be either `kde`, `cumulative`, or `scatter`")
 
+    if colors is None:
+        colors = ["C0", "k", "C1"]
+
+    if isinstance(colors, str):
+        raise TypeError("colors should be a list with 3 items.")
+
+    if len(colors) != 3:
+        raise ValueError("colors should be a list with 3 items.")
+
+    if color is not None:
+        warnings.warn("color has been deprecated in favor of colors", FutureWarning)
+        colors[0] = color
+
     if data_pairs is None:
         data_pairs = {}
 
@@ -235,6 +258,9 @@ def plot_ppc(
     if coords is None:
         coords = {}
 
+    if labeller is None:
+        labeller = BaseLabeller()
+
     if random_seed is not None:
         np.random.seed(random_seed)
 
@@ -251,8 +277,7 @@ def plot_ppc(
         or num_pp_samples > total_pp_samples
     ):
         raise TypeError(
-            "`num_pp_samples` must be an integer between 1 and "
-            + "{limit}.".format(limit=total_pp_samples)
+            "`num_pp_samples` must be an integer between 1 and " + f"{total_pp_samples}."
         )
 
     pp_sample_ix = np.random.choice(total_pp_samples, size=num_pp_samples, replace=False)
@@ -299,13 +324,14 @@ def plot_ppc(
         pp_sample_ix=pp_sample_ix,
         kind=kind,
         alpha=alpha,
-        color=color,
+        colors=colors,
         jitter=jitter,
         textsize=textsize,
         mean=mean,
         observed=observed,
         total_pp_samples=total_pp_samples,
         legend=legend,
+        labeller=labeller,
         group=group,
         animation_kwargs=animation_kwargs,
         num_pp_samples=num_pp_samples,
